/**
	Program to produce an equalized image 

	(using the histogram equalization program)
	Input image must be in TIFF format

	Fraga. Jun 4, 2001
**/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "matrix.h"
#include "tiff.h"

MATRIX * do_histogram ( MATRIX * IN )
{
	MATRIX *HIST;
	int *histo, i, j;
	byte *iptr;

	HIST = matrix_allocate( 1, 256, sizeof(int) );
	if ( HIST==NULL ) {
		fprintf( stderr, "ERROR allocating memory for histogram\n" );
		return NULL;
	}
	histo = (int *)HIST->ptr[0];
	for ( i=0; i<256; i++ ) histo[i] = 0;

	for ( i=0; i<IN->rows; i++ ) {
		iptr = (byte *)IN->ptr[i];
		for ( j=0; j<IN->cols; j++ )
			histo[ *iptr++ ] ++;
	}
	return( HIST );
}
 
void print_histogram ( MATRIX * IN )
{
	int *histo, i;

	histo = (int *)IN->ptr[0];

	for ( i=0; i<256; i++ )
		printf( "%d %d\n", i,  histo[i] );
}

MATRIX * do_histo_equalization( MATRIX * IN ) 
{
	MATRIX *OUT, *HIST;
	float p_f[256];
	byte s[256];
	int i, j, sum, *paux;
	byte *pin, *pout;

	if ( (HIST = do_histogram( IN )) == NULL) return NULL;	
// 	print_histogram ( HIST );

	OUT = matrix_allocate( IN->rows, IN->cols, 1 );
	if ( OUT == NULL ) {
		fprintf( stderr,"ERROR allocating memory for output equalized image\n");
		return NULL;
	}

	paux = (int *)HIST -> ptr[0];
	sum = 0;
	for ( i=0; i<256; i++ ){
		sum += paux[i];
		p_f[i] = (float)sum/(IN->rows * IN->cols);
	}  
	for ( i=0; i<256; i++ ){
		s[i] = (byte )( 255.0*p_f[ i ] ); 			
		// printf( "%d %d\n", i,  s[i] );
	}

	for ( i=0; i<IN->rows; i++ ) {
		pin = (byte *)IN->ptr[i];
		pout = (byte *)OUT->ptr[i];

		for ( j=0; j<IN->cols; j++ ) 
			pout[j] = s [ pin[j] ]; 			
			// pout[j] = pin[j]; 			
	}

	return OUT;
}

int do_work( char *namein ) 
{
	MATRIX *IN, *EQ;
	FILEINFO finfo;
	int i, j;
	byte *iptr;
	

	IN = read_tiff( &finfo, namein ); 
	if ( IN==NULL ) {
		fprintf( stderr, "ERROR reading %s file\n", namein );
		return -1;
	}
	if ( IN->type != 5 ) {
		fprintf( stderr, "ERROR Bad format. Image must be in 256 grey tones\n");
		return -1;
	}

	EQ = do_histo_equalization( IN );
	if ( EQ == NULL ) {
		fprintf( stderr, "ERROR doing histogram equalization\n");
	 	return -1;
	}
	EQ->type=5;
	if ( write_tiff( EQ, "equalized.tif" ) != GOOD_WRITE )  
        return -1;

	matrix_free( IN );
	matrix_free( EQ );

	return 0;
}

int main( int argc, char *argv[] )
{
	char namein[32];

	if ( argc < 2 ) { 
		fprintf( stderr, "Use: %s image.tif\n", argv[0] );
		return -1;
	}

	strncpy( namein, argv[1], 32 );

	if ( do_work( namein ) >= 0 ) {
		fprintf( stderr, "OK!!!\n" );
		return 0;
	}
	else {
		fprintf( stderr, "ERROR: Something is wrong!!!\n" );
		exit(1);
	}
	return 0;
}
