/**
	Program to do the spectrum image of the
	FFT  of the inout image

	Fraga. May 30, 2001
**/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "matrix.h"
#include "tiff.h"
#include "fft.h"

void rescale( float **imgin, byte **imgout, int rows, int cols )
{
	float fmin, fmax, val, scale;
	int i, j;

	/** We rescale the dinamic range from 0 to 255 **/
	fmin = 1e30;
	fmax = -1e30;
	for ( i=0; i<rows; i++ ){
		for ( j=0; j<cols; j++ ){
			val = imgin[i][j];
			if ( fmin > val ) fmin = val;
			if ( fmax < val ) fmax = val;
		}
	}

	scale = 255.0/(fmax-fmin);
	for ( i=0; i<rows; i++ ){
		for ( j=0; j<cols; j++ )
			imgout[i][j] = (byte)(scale*( imgin[i][j] - fmin ) + 0.5 ); 
	}
}

int do_work( char *namein ) 
{
	MATRIX *IN;
	MATRIX *FFTIMG;
	MATRIX *OUT;
	FILEINFO finfo;
	int i, j, signo;
	byte *iptr;
	float *fptr;
	

	IN = read_tiff( &finfo, namein ); 
	if ( IN==NULL ) {
		fprintf( stderr, "ERROR reading %s file\n", namein );
		return -1;
	}
	FFTIMG = matrix_allocate( IN->rows, IN->cols + 2, sizeof(float) );

	/** We copy the readed image to the new float matrix **/
	/** The variable signo is for move the FT a half of a period **/
	signo = -1;
	for ( i=0; i<IN->rows; i++ ) {
		iptr = (byte *)IN->ptr[i];
		fptr = (float *)FFTIMG->ptr[i];
		for ( j=0; j<IN->cols; j++ ) {
			signo *= -1;
			*fptr++ = *iptr++ * signo;
	//		*fptr++ = *iptr++;
		}
		signo *= -1;
	}

	/** Now we calculate the FFT **/
	image_fft ( (float **)FFTIMG->ptr, IN->rows , IN->cols, DIRECT );

	OUT = matrix_allocate( IN->rows, IN->cols, sizeof(float) );
	spectrum( (float **)FFTIMG->ptr, (float **)OUT->ptr, IN->rows, IN->cols );

	rescale( (float **)OUT->ptr, (byte **)IN->ptr, IN->rows, IN->cols );
	
	/** So, we store the spectrum **/
	if ( write_tiff( IN, "spectrum.tif" ) != GOOD_WRITE )
		return -1;
	
	matrix_free( IN );
	matrix_free( FFTIMG );
	matrix_free( OUT );
}

int main( int argc, char *argv[] )
{
	char namein[32];

	if ( argc < 2 ) { 
		fprintf( stderr, "Use: %s image.tif\n", argv[0] );
		return -1;
	}

	strncpy( namein, argv[1], 32 );

	if ( do_work( namein ) >= 0 ) {
		fprintf( stderr, "OK!!!\n" );
		return 0;
	}
	else {
		fprintf( stderr, "ERROR: Something is wrong!!!\n" );
		exit(1);
	}
	return 0;
}
